

/* I2C LCD with Arduino example code. More info: https://www.makerguides.com */

// Include the libraries:
// LiquidCrystal_I2C.h: https://github.com/johnrickman/LiquidCrystal_I2C
#include <Wire.h> // Library for I2C communication
#include <LiquidCrystal_I2C.h> // Library for LCD

// Wiring: SDA pin is connected to A4 and SCL pin to A5.
// Connect to LCD via I2C, default address 0x27 (A0-A2 not jumpered)
LiquidCrystal_I2C lcd = LiquidCrystal_I2C(0x27, 16, 2); // Change to (0x27,20,4) for 20x4 LCD.

//initialize all variables
const int buzzer1_press = 11;
const int buzzer2_press = 10;
const int buzzer3_press = 9;
const int buzzer4_press = 8;
const int host_press = 12;
const int led_buzz1 = 3;
const int led_buzz2 = 4;
const int led_buzz3 = 5;
const int led_buzz4 = 6;
const int led_host = 7;
const int led_debug = 13;

int buzz1_state = 0;
int buzz2_state = 0;
int buzz3_state = 0;
int buzz4_state = 0;
int host_state = 0;
int host_ready = 0;
boolean penalty1 = 0;
boolean penalty2 = 0;
boolean penalty3 = 0;
boolean penalty4 = 0;
int t_penalty1 = 0;
int t_penalty2 = 0;
int t_penalty3 = 0;
int t_penalty4 = 0;
int time = 0;

void setup() {
  //setup our pins
  pinMode(buzzer1_press, INPUT);
  pinMode(buzzer2_press, INPUT);
  pinMode(buzzer3_press, INPUT);
  pinMode(buzzer4_press, INPUT);
  pinMode(host_press, INPUT);
  pinMode(led_buzz1, OUTPUT);
  pinMode(led_buzz2, OUTPUT);
  pinMode(led_buzz3, OUTPUT);
  pinMode(led_buzz4, OUTPUT);
  pinMode(led_host, OUTPUT);
  pinMode(led_debug, OUTPUT);

  digitalWrite(led_buzz1, LOW);
  digitalWrite(led_buzz2, LOW);
  digitalWrite(led_buzz3, LOW);
  digitalWrite(led_buzz4, LOW);
  digitalWrite(led_host, LOW);
  digitalWrite(led_debug,LOW);

  // Initiate the LCD screen, print the welcome screen:
  lcd.init();
  lcd.backlight();
  lcd.setCursor(0, 0); // Set the cursor on the third column and first row.
  lcd.print("Welcome to"); // Print the string "Hello World!"
  lcd.setCursor(4, 1); //Set the cursor on the third column and the second row (counting starts at 0!).
  lcd.print("Jeopardy!");
  delay(5000);//wait 5 seconds
  lcd.clear();
  //hold the next message as we go into the loop()
  lcd.setCursor(0, 0);
  lcd.print("Host, press");
  lcd.setCursor(0, 1);
  lcd.print("Reset to begin");
  host_ready = wait_for_press(host_press);
  if (host_ready)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Ready for Q");
    lcd.setCursor(0, 1);
    //now that the host is ready may read the question. Hit host button again after reading Q to allow
    //contestants to buzz in
    host_ready = 0; //reset host button to allow second press shortly
    delay(1000);
  }
}

void loop() {
    buzz_in();

}

int wait_for_press(int a) {

  int ready_time=0;
  int button_pressed = 1;
  while(ready_time == 0)
  {
  button_pressed = digitalRead(a); //check if button has been pressed
  penalty_check();
  penalty_remove();
  if (button_pressed == LOW)
  {
    ready_time = 1;
    return ready_time;
  }
  }
}

void buzz_in() {

  host_ready = wait_for_press(host_press);
  digitalWrite(led_host,HIGH);
  while(host_ready){
    penalty_remove();
    buzz1_state = digitalRead(buzzer1_press); 
    buzz2_state = digitalRead(buzzer2_press); 
    buzz3_state = digitalRead(buzzer3_press); 
    buzz4_state = digitalRead(buzzer4_press);
    if(buzz1_state == LOW && !penalty1)
    {
      digitalWrite(led_buzz1,HIGH);
      //update the screen
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("1st: Player 1!!");
      host_ready = 0;
    }
    if(buzz2_state == LOW && !penalty2)
    {
      digitalWrite(led_buzz2,HIGH);
      //update the screen
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("1st: Player 2!!");
      host_ready = 0;
    }
    if(buzz3_state == LOW && !penalty3)
    {
      digitalWrite(led_buzz3,HIGH);
      //update the screen
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("1st: Player 3!!");
      host_ready = 0;
    }
    if(buzz4_state == LOW && !penalty4)
    {
      digitalWrite(led_buzz4,HIGH);
      //update the screen
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("1st: Player 4!!");
      host_ready = 0;
    }
    
  }
  //next we need to check if first place has been found, if so start looking for second place
  //lastly, we need to be able to reset back to where we started, reset variables to where they need to be and display "ready for Q" again. repeat buzz_in();
  host_ready = wait_for_press(host_press);
  if (host_ready)
  {
    digitalWrite(led_buzz1,LOW);
    digitalWrite(led_buzz2,LOW);
    digitalWrite(led_buzz3,LOW);
    digitalWrite(led_buzz4,LOW);
    digitalWrite(led_host,LOW);
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Ready for Q");
    lcd.setCursor(0, 1);
    //now that the host is ready may read the question. Hit host button again after reading Q to allow
    //contestants to buzz in
    host_ready = 0; //reset host button to allow second press shortly
    delay(2000);
  }
  
}

void penalty_check(){
    buzz1_state = digitalRead(buzzer1_press); //check these to see if we need to penalize for early press
    buzz2_state = digitalRead(buzzer2_press); //check these to see if we need to penalize for early press
    buzz3_state = digitalRead(buzzer3_press); //check these to see if we need to penalize for early press
    buzz4_state = digitalRead(buzzer4_press); //check these to see if we need to penalize for early press
    if(buzz1_state == 0)
    {
      penalty1=1;
      t_penalty1=millis();
    }
    if(buzz2_state == 0)
    {
      penalty2=1;
      t_penalty2=millis();
    }
    if(buzz3_state == 0)
    {
      penalty3=1;
      t_penalty3=millis();
    }
    if(buzz4_state == 0)
    {
      penalty4=1;
      t_penalty4=millis();
    }

}

void penalty_remove(){
    time = millis();
    if(penalty1 == 1 && time-t_penalty1 > 2000)
    {
      penalty1=0;
      t_penalty1=0;
    }
    if(penalty2 == 1 && time-t_penalty2 > 2000)
    {
      penalty2=0;
      t_penalty2=0;
    }
    if(penalty3 == 1 && time-t_penalty3 > 2000)
    {
      penalty3=0;
      t_penalty3=0;
    }
    if(penalty4 == 1 && time-t_penalty4 > 2000)
    {
      penalty4=0;
      t_penalty4=0;
    }
}
